@@ -1,3 +1,8 @@
+//! # Basic module
+//!
+//! Basic module includes struct and methods related to basic macros which are technically function
+//! pointers.
+
 use std::array::IntoIter;
 use std::io::Write;
 use std::fs::OpenOptions;
@@ -6,18 +11,33 @@ use std::iter::FromIterator;
 use std::path::PathBuf;
 use std::process::Command;
 use crate::error::RadError;
-use crate::arg_parser::{ArgParser, GreedyState};
+use crate::arg_parser::ArgParser;
 use crate::consts::MAIN_CALLER;
 use regex::Regex;
 use crate::utils::Utils;
-use crate::processor::Processor;
+use crate::processor::{Processor, auth::{AuthState, AuthType}};
 #[cfg(feature = "csv")]
 use crate::formatter::Formatter;
 #[cfg(feature = "lipsum")]
 use lipsum::lipsum;
 
-// Args, greediness, processor
-type MacroType = fn(&str, bool ,&mut Processor) -> Result<String, RadError>;
+/// Type signature of basic macros
+///
+/// This is in order of args, greediness, processor's mutable reference
+///
+/// # Example
+///
+/// ```rust
+/// fn demo(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
+///     let mut medium = String::new();
+///     // Some logics go here
+///     Ok(medium)
+/// }
+///
+/// // ... While building a processor ...
+/// processor.add_basic_rules(vec![("test", test as MacroType)]);
+/// ```
+pub type MacroType = fn(&str, bool ,&mut Processor) -> Result<String, RadError>;
 
 #[derive(Clone)]
 pub struct BasicMacro {
@@ -25,9 +45,19 @@ pub struct BasicMacro {
 }
 
 impl BasicMacro {
+    /// Creates empty map
+    pub fn empty() -> Self {
+        Self {
+            macros: HashMap::new(),
+        }
+    }
+
     /// Creates new basic macro hashmap
+    ///
+    /// Optional macros are included only when a feature is enabled
     pub fn new() -> Self {
         // Create hashmap of functions
+        #[allow(unused_mut)]
         let mut map = HashMap::from_iter(IntoIter::new([
             ("regex".to_owned(), BasicMacro::regex_sub as MacroType),
             ("trim".to_owned(), BasicMacro::trim as MacroType),
@@ -81,6 +111,11 @@ impl BasicMacro {
         Self { macros : map }
     }
 
+    /// Add new basic rule
+    pub fn add_new_rule(&mut self, name: &str, macro_ref: MacroType) {
+        self.macros.insert(name.to_owned(), macro_ref);
+    }
+
     /// Check if a given macro exists
     ///
     /// # Arguments
@@ -90,6 +125,21 @@ impl BasicMacro {
         self.macros.contains_key(name)
     }
 
+    /// Check file authority
+    fn is_granted(name:&str, auth_type: AuthType, processor: &mut Processor) -> Result<bool, RadError> {
+        match processor.get_auth_state(&auth_type) {
+            AuthState::Restricted => {
+                processor.log_error(&format!("\"{}\" not allowed.\"{:?}\" permission is required", name, auth_type))?;
+                Ok(false)
+            }
+            AuthState::Warn => {
+                processor.log_warning(&format!("\"{}\" was called with \"{:?}\" permission", name, auth_type))?;
+                Ok(true)
+            }
+            AuthState::Open => Ok(true),
+        }
+    }
+
     /// Call a macro
     ///
     /// # Arguments
@@ -99,8 +149,6 @@ impl BasicMacro {
     /// * `greedy` - Whether macro should interpret arguments greedily
     /// * `processor` - Processor instance to execute macro
     pub fn call(&mut self, name : &str, args: &str,greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
-        // TODO
-        // Check if this code is necessary
         if let Some(func) = self.macros.get(name) {
             // Print out macro call result
             let result = func(args, greedy, processor)?;
@@ -185,8 +233,7 @@ impl BasicMacro {
         if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
             let formula = &args[0];
             let result = evalexpr::eval(formula)?;
-            // TODO
-            // Enable floating points length (or something similar)
+            // TODO Enable floating points length (or something similar)
             Ok(result.to_string())
         } else {
             Err(RadError::InvalidArgument("Eval requires an argument".to_owned()))
@@ -267,6 +314,9 @@ impl BasicMacro {
     ///
     /// $include(path)
     fn include(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
+        if !Self::is_granted("include", AuthType::FIN,processor)? {
+            return Ok(String::new());
+        }
         if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
             let raw = Utils::trim(&args[0])?;
             let file_path = std::path::Path::new(&raw);
@@ -311,13 +361,18 @@ impl BasicMacro {
     ///
     /// This calls via 'CMD \C' in windows platform while unix call is operated without any mediation.
     ///
+    /// Syscmd is always greedy
+    ///
     /// # Usage
     ///
     /// $syscmd(system command -a arguments)
-    fn syscmd(args: &str, greedy: bool,_: &mut Processor) -> Result<String, RadError> {
-        if let Some(args_content) = ArgParser::new().args_with_len(args, 1, greedy) {
+    fn syscmd(args: &str, _: bool,p: &mut Processor) -> Result<String, RadError> {
+        if !Self::is_granted("syscmd", AuthType::CMD,p)? {
+            return Ok(String::new());
+        }
+        if let Some(args_content) = ArgParser::new().args_with_len(args, 1, true) {
             let source = &args_content[0];
-            let arg_vec = ArgParser::new().args_to_vec(&source, ' ', GreedyState::None);
+            let arg_vec = source.split(' ').collect::<Vec<&str>>();
 
             let output = if cfg!(target_os = "windows") {
                 Command::new("cmd")
@@ -343,9 +398,11 @@ impl BasicMacro {
 
     /// Print content according to given condition
     /// 
+    /// If macro's second argument is evaluated twice.
+    ///
     /// # Usage 
     ///
-    /// $if(evaluation, ifstate)
+    /// $if(evaluation, \*ifstate*\)
     fn if_cond(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
         if let Some(args) = ArgParser::new().args_with_len(args, 2, greedy) {
             let boolean = &args[0];
@@ -355,12 +412,12 @@ impl BasicMacro {
             let trimmed_cond = Utils::trim(boolean)?;
             if let Ok(cond) = trimmed_cond.parse::<bool>() {
                 if cond { 
-                    let result = processor.parse_chunk(0, &MAIN_CALLER.to_owned(), if_state)?;
+                    let result = processor.parse_chunk_args(0, &MAIN_CALLER.to_owned(), if_state)?;
                     return Ok(result); 
                 }
             } else if let Ok(number) = trimmed_cond.parse::<i32>() {
                 if number != 0 { 
-                    let result = processor.parse_chunk(0, &MAIN_CALLER.to_owned(), if_state)?;
+                    let result = processor.parse_chunk_args(0, &MAIN_CALLER.to_owned(), if_state)?;
                     return Ok(result); 
                 }
             } else {
@@ -375,9 +432,11 @@ impl BasicMacro {
 
     /// Print content according to given condition
     /// 
+    /// Ifelse second and third arguemtns are evaluated twice.
+    ///
     /// # Usage 
     ///
-    /// $ifelse(evaluation, ifstate, elsestate)
+    /// $ifelse(evaluation, \*ifstate*\, \*elsestate*\)
     fn ifelse(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
         if let Some(args) = ArgParser::new().args_with_len(args, 3, greedy) {
             let boolean = &args[0];
@@ -387,12 +446,12 @@ impl BasicMacro {
             let trimmed_cond = Utils::trim(boolean)?;
             if let Ok(cond) = trimmed_cond.parse::<bool>() {
                 if cond { 
-                    let result = processor.parse_chunk(0, &MAIN_CALLER.to_owned(), if_state)?;
+                    let result = processor.parse_chunk_args(0, &MAIN_CALLER.to_owned(), if_state)?;
                     return Ok(result); 
                 }
             } else if let Ok(number) = trimmed_cond.parse::<i32>() {
                 if number != 0 { 
-                    let result = processor.parse_chunk(0, &MAIN_CALLER.to_owned(), if_state)?;
+                    let result = processor.parse_chunk_args(0, &MAIN_CALLER.to_owned(), if_state)?;
                     return Ok(result); 
                 }
             } else {
@@ -401,7 +460,7 @@ impl BasicMacro {
 
             // Else state
             let else_state = &args[2];
-            let result = processor.parse_chunk(0, &MAIN_CALLER.to_owned(), else_state)?;
+            let result = processor.parse_chunk_args(0, &MAIN_CALLER.to_owned(), else_state)?;
             return Ok(result);
         } else {
             Err(RadError::InvalidArgument("ifelse requires three argument".to_owned()))
@@ -433,7 +492,7 @@ impl BasicMacro {
 
     /// Undefine a macro 
     ///
-    /// 'Define' cannot be undefined because it is not actually a macro 
+    /// 'Define' and 'BR' cannot be undefined because it is not actually a macro 
     ///
     /// # Usage
     ///
@@ -451,9 +510,11 @@ impl BasicMacro {
 
     /// Loop around given values and substitute iterators  with the value
     ///
+    /// Foreach's second macro is evaluated twice.
+    ///
     /// # Usage 
     ///
-    /// $foreach(\*a,b,c*\,$:)
+    /// $foreach(\*a,b,c*\,\*$:*\)
     fn foreach(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
         if let Some(args) = ArgParser::new().args_with_len(args, 2, greedy) {
             let mut sums = String::new();
@@ -461,7 +522,7 @@ impl BasicMacro {
             let loopable = &args[0];
 
             for value in loopable.split(',') {
-                let result = processor.parse_chunk(0, &MAIN_CALLER.to_owned(), &target.replace("$:", value))?;
+                let result = processor.parse_chunk_args(0, &MAIN_CALLER.to_owned(), &target.replace("$:", value))?;
                 sums.push_str(&result);
             }
             Ok(sums)
@@ -472,9 +533,11 @@ impl BasicMacro {
 
     /// For loop around given min, max value and finally substitue iterators with value
     ///
+    /// Forloop's second macro is evaluated twice.
+    ///
     /// # Usage
     ///
-    /// $forloop(1,5,$:)
+    /// $forloop(1,5,\*$:*\)
     fn forloop(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
         if let Some(args) = ArgParser::new().args_with_len(args, 3, greedy) {
             let mut sums = String::new();
@@ -490,7 +553,7 @@ impl BasicMacro {
             } else { return Err(RadError::InvalidArgument("Forloop's min value should be non zero positive integer".to_owned())); }
             
             for value in min..=max {
-                let result = processor.parse_chunk(0, &MAIN_CALLER.to_owned(), &expression.replace("$:", &value.to_string()))?;
+                let result = processor.parse_chunk_args(0, &MAIN_CALLER.to_owned(), &expression.replace("$:", &value.to_string()))?;
                 sums.push_str(&result);
             }
 
@@ -514,7 +577,7 @@ impl BasicMacro {
 
             let result = Formatter::csv_to_macros(macro_name, macro_data, &processor.newline)?;
             // This is necessary
-            let result = processor.parse_chunk(0, &MAIN_CALLER.to_owned(), &result)?;
+            let result = processor.parse_chunk_args(0, &MAIN_CALLER.to_owned(), &result)?;
             Ok(result)
         } else {
             Err(RadError::InvalidArgument("From requires two arguments".to_owned()))
@@ -579,7 +642,10 @@ impl BasicMacro {
     /// # Usage
     ///
     /// $env(SHELL)
-    fn get_env(args: &str, _: bool, _: &mut Processor) -> Result<String, RadError> {
+    fn get_env(args: &str, _: bool, p : &mut Processor) -> Result<String, RadError> {
+        if !Self::is_granted("env", AuthType::ENV,p)? {
+            return Ok(String::new());
+        }
         let out = std::env::var(args)?;
         Ok(out)
     }
@@ -639,6 +705,8 @@ impl BasicMacro {
 
     /// Rename macro rule to other name
     ///
+    /// Define and BR can't be renamed.
+    ///
     /// # Usage
     ///
     /// $rename(name,target)
@@ -656,6 +724,8 @@ impl BasicMacro {
 
     /// Append content to a macro
     ///
+    /// Only custom macros can be appended.
+    ///
     /// # Usage
     ///
     /// $append(macro_name,Content)
@@ -734,6 +804,7 @@ impl BasicMacro {
         }
     }
 
+    // TODO Make pause should be a logic branch not a basic macro?
     /// Pause every macro expansion
     ///
     /// Only other pause call is evaluated
@@ -768,6 +839,10 @@ impl BasicMacro {
     ///
     /// $tempout(true,Content)
     fn temp_out(args: &str, greedy: bool, p: &mut Processor) -> Result<String, RadError> {
+        if !Self::is_granted("temput", AuthType::FOUT,p)? {
+            return Ok(String::new());
+        }
+
         if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
             let content = &args[0];
             p.get_temp_file().write_all(content.as_bytes())?;
@@ -782,7 +857,10 @@ impl BasicMacro {
     /// # Usage
     ///
     /// $fileout(true,file_name,Content)
-    fn file_out(args: &str, greedy: bool, _: &mut Processor) -> Result<String, RadError> {
+    fn file_out(args: &str, greedy: bool, p: &mut Processor) -> Result<String, RadError> {
+        if !Self::is_granted("fileout", AuthType::FOUT,p)? {
+            return Ok(String::new());
+        }
         if let Some(args) = ArgParser::new().args_with_len(args, 3, greedy) {
             let truncate = &args[0];
             let file_name = &args[1];
@@ -819,6 +897,9 @@ impl BasicMacro {
     ///
     /// $tempin()
     fn temp_include(_: &str, _: bool, processor: &mut Processor) -> Result<String, RadError> {
+        if !Self::is_granted("tempin", AuthType::FIN,processor)? {
+            return Ok(String::new());
+        }
         let file = processor.get_temp_path().to_owned();
         processor.set_sandbox();
         Ok(processor.from_file(&file)?)
@@ -832,14 +913,17 @@ impl BasicMacro {
     ///
     /// $redir(true) 
     /// $redir(false) 
-    fn temp_redirect(args: &str, _: bool, processor: &mut Processor) -> Result<String, RadError> {
+    fn temp_redirect(args: &str, _: bool, p: &mut Processor) -> Result<String, RadError> {
+        if !Self::is_granted("redir", AuthType::FOUT,p)? {
+            return Ok(String::new());
+        }
         if let Some(args) = ArgParser::new().args_with_len(args, 1, false) {
             let toggle = if let Ok(toggle) =Utils::is_arg_true(&args[0]) { 
                 toggle
             } else {
                 return Err(RadError::InvalidArgument("Redir's agument should be valid boolean value".to_owned()));
             };
-            processor.redirect = toggle;
+            p.redirect = toggle;
             Ok(String::new())
         } else {
             Err(RadError::InvalidArgument("Redir requires an argument".to_owned()))
@@ -852,6 +936,9 @@ impl BasicMacro {
     ///
     /// $tempto(file_name)
     fn set_temp_target(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
+        if !Self::is_granted("tempto", AuthType::FOUT,processor)? {
+            return Ok(String::new());
+        }
         if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
             processor.set_temp_file(&PathBuf::from(std::env::temp_dir()).join(&args[0]));
             Ok(String::new())
