@@ -154,7 +154,7 @@ impl Cli {
     /// Creates argument template wich clap macro
     fn args_builder() -> clap::ArgMatches {
         clap_app!(rad =>
-            (version: "0.8.0")
+            (version: "0.8.1")
             (author: "Simon Creek <simoncreek@tutanota.com>")
             (about: "R4d is a modern macro processor made with rust")
             (@arg FILE: ... "Files to execute processing")
@@ -170,8 +170,8 @@ impl Cli {
             (@arg interactive: -i --interactive "Use interactive debug mode")
             (@arg combination: -c "Read from both stdin and file inputs")
             (@arg discard: -D --discard "Discard output without prin out")
-            (@arg allow: -a +takes_value "Allow permission (io|cmd|env)")
-            (@arg allow_warn: -w +takes_value "Allow permission with warnings (io|cmd|env)")
+            (@arg allow: -a +takes_value "Allow permission (fin|fout|cmd|env)")
+            (@arg allow_warn: -w +takes_value "Allow permission with warnings (fin|fout|cmd|env)")
             (@arg allow_all: -A "Allow all permission")
             (@arg allow_all_warn: -W "Allow all permission with warning")
             (@arg silent: -s "Supress error and warning")
